import { CacheProvider, css, Global } from '@emotion/react';
import CssBaseline from '@mui/material/CssBaseline';
import { ThemeProvider } from '@mui/material/styles';
import { FC, PropsWithChildren, useMemo } from 'react';

import nuskinTheme from '../themes/genome-foundation';
import { generateGlobalColorCss, generateTextAlignCss } from '../utils';
import { defaultCacheKey, getCacheKey } from './getCacheKey';
import { NextGenThemeProviderProps } from './types';
import { useAppTheme } from './useAppTheme';

type PropType = FC<PropsWithChildren<NextGenThemeProviderProps>>;

export const NextGenThemeProvider: PropType = (props) => {
    const { isSSR = false, children, cacheKey = defaultCacheKey, theme = nuskinTheme, mode } = props;

    const { appMuiTheme, isDarkMode } = useAppTheme({ mode, theme });

    const cache = useMemo(() => getCacheKey(cacheKey), [cacheKey]);

    const GlobalCss = useMemo(() => {
        return (
            <>
                <CssBaseline />
                <Global
                    styles={css`
                        ${generateTextAlignCss()}
                        ${generateGlobalColorCss(theme)}
                    `}
                />
            </>
        );
    }, [theme]);

    const Content = (
        <ThemeProvider theme={appMuiTheme}>
            {GlobalCss}
            <main className={`theme-wrapper ${isDarkMode ? 'dark' : 'light'}`}>{children}</main>
        </ThemeProvider>
    );

    if (isSSR) {
        return <CacheProvider value={cache}>{Content}</CacheProvider>;
    }
    return Content;
};

export default NextGenThemeProvider;
