import type { Palette, PaletteKeys } from '@mui/material/styles';

import { ComponentConfig } from '.';
import { GenomeColorType } from '../genome-colors/AllColors';

export type PaletteThemeColors =
    | 'primary'
    | 'neutral'
    | 'accentColorA'
    | 'accentColorB'
    | 'accentColorC'
    | 'accentColorD'
    | 'secondary'
    | 'success'
    | 'warning'
    | 'error'
    | 'info';
export const PaletteColors: PaletteKeys[] = [
    'primary',
    'neutral',
    'accentColorA',
    'accentColorB',
    'accentColorC',
    'accentColorD',
    'secondary',
    'success',
    'warning',
    'error',
    'info',
];

export const AppColorGroups = [...PaletteColors, 'custom'];

export namespace FoundationTheme {
    export type Mode = 'light' | 'dark';

    export type HexColor = `#${string}`;

    export type CommonColors = {
        white: string;
        black: string;
    };

    type ColorGroupPrimary = {
        primary?: {
            black: HexColor;
            white: HexColor;
            gray: HexColor;
            dark: HexColor;
        };
    };

    export type PaletteColors = keyof Palette;

    export type AppColors = (typeof AppColorGroups)[number];

    export type MainDarkColors = { main: string; dark?: string };

    export type Colors = Record<AppColors, Record<string, string | MainDarkColors> & ColorGroupPrimary>;

    export interface ButtonState {
        hovered?: string;
        hoveredText?: string;
        pressed?: string;
        pressedText?: string;
        focused?: string;
        focusedText?: string;
        disabled?: string;
        disabledText?: string;
    }

    export type PaletteColorType = {
        main: string;
        contrastText: string;
        buttonState?: ButtonState;
        darkMode?: {
            main: string;
            contrastText: string;
            buttonState?: ButtonState;
        };
    };

    export type ThemeConfig = {
        components?: ComponentConfig;
    };

    export type Theme = {
        palette?: Palette;
        colors?: Colors;
        config?: ThemeConfig;
        genomeColors?: GenomeColorType;
    };
    export type Themed = { theme: FoundationTheme.Theme };
}
