import { FoundationTheme } from '../../types/foundationTheme';
import { AllColorKeyType, AllColors } from '../AllColors';
import { getGenomeColor } from '../ColorFunctions';

describe('ColorFunction', () => {
    it('1. should return the correct color for a valid colorName in light mode', () => {
        const theme: FoundationTheme.Theme = {
            palette: {
                mode: 'light',
                primary: {
                    main: '#121212',
                    contrastText: '#f5f5f5',
                },
            },
            genomeColors: AllColors,
        };
        const colorName: AllColorKeyType = 'N100';
        const result = getGenomeColor(theme, colorName);
        expect(result).toBe('#252525');
    });
    it('2. should return the correct color for a valid colorName in dark mode', () => {
        const theme: FoundationTheme.Theme = {
            palette: {
                mode: 'dark',
                primary: {
                    main: '#121212',
                    contrastText: '#f5f5f5',
                },
            },
            genomeColors: AllColors,
        };
        const colorName: AllColorKeyType = 'N100';
        const result = getGenomeColor(theme, colorName);
        expect(result).toBe('#FFFFFF');
    });
    it('3. should return undefined for an invalid colorName', () => {
        const theme: FoundationTheme.Theme = {
            palette: {
                mode: 'dark',
                primary: {
                    main: '#121212',
                    contrastText: '#f5f5f5',
                },
            },
            genomeColors: AllColors,
        };
        const colorName: AllColorKeyType = 'invalid' as any;
        const result = getGenomeColor(theme, colorName);
        expect(result).toBeUndefined();
    });
    it('4. should return undefined when theme is undefined', () => {
        const theme = undefined;
        const colorName: AllColorKeyType = 'N100';
        const result = getGenomeColor(theme as any, colorName);
        expect(result).toBeUndefined();
    });
    it('5. should return light mode color when palette is undefined', () => {
        const theme: FoundationTheme.Theme = {
            genomeColors: AllColors,
        };
        const colorName: AllColorKeyType = 'N100';
        const result = getGenomeColor(theme, colorName);
        expect(result).toBe('#252525');
    });
    it('6. should be able to forceMode:dark', () => {
        const theme: FoundationTheme.Theme = {
            genomeColors: AllColors,
        };
        const colorName: AllColorKeyType = 'N100';
        const result = getGenomeColor(theme, colorName, 'dark');
        expect(result).toBe('#FFFFFF');
    });
    it('7. should be able to forceMode:dark', () => {
        const theme: FoundationTheme.Theme = {
            genomeColors: AllColors,
        };
        const colorName: AllColorKeyType = 'N100';
        const result = getGenomeColor(theme, colorName, 'light');
        expect(result).toBe('#252525');
    });
});
